/*
 * file CqAttachmentFolder.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqAttachmentFolder
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.Folder;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

/**
 * A folder that contains the attachments associated with an attachment field of
 * a record and, hence, the value of such a record field. Attachments are added
 * to a record by creating an attachment resource in an CqAttachmentFolder. They
 * are removed by deleting the attachment resources from their respective
 * attachment folders.
 * 
 * <p>
 * Attachment folders can neither be created nor deleted independent of the
 * record with which they are associated. They are the one and only value of
 * their attachment field for the lifetime of that field. Only their contents
 * may be altered. And their content is altered by executing
 * {@link CqAttachment#doCreateAttachment(String, Feedback, java.util.List) 
 * doCreateAttachment} and
 * {@link CqAttachment#doUnbindAll(Feedback) doUnbindAll} operations on the
 * attachment resources that are contained by the attachment folder.
 * <p>
 * The user-friendly specification for the location of an attachment folder has the form
 * <pre>
 *  <b>cq.record:</b><i>&lt;record-type&gt;</i>/<i>&lt;record-id&gt;</i>/<i>&lt;field-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 * where the /<i>&lt;field-name&gt;</i> segment identifies a field of type
 * {@link CqFieldValue.ValueType#ATTACHMENT_LIST}
 */
public interface CqAttachmentFolder extends CqUserDbMember, Folder {
    /** 
     * The members of this folder. The attachments that are associated with
     * this folder's parent record field. A more-strongly-typed version of the
     * the {@link Folder#CHILD_LIST} property.
     */
    PropertyName<ResourceList<CqAttachment>> ATTACHMENT_LIST = 
        new PropertyName<ResourceList<CqAttachment>>(PROPERTY_NAMESPACE, "attachment-list");
    
    /**
     * Returns the value of the {@link #ATTACHMENT_LIST ATTACHMENT_LIST}
     * property as defined by this proxy.
     * 
     * @return A List of CqAttachment proxies for the attachments in this
     *         attachment folder.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ATTACHMENT_LIST ATTACHMENT_LIST} property.
     */
    ResourceList<CqAttachment> getAttachmentList() throws WvcmException;

    /** The record field that contains this attachment folder as its value. */
    PropertyName<CqFieldValue<CqAttachmentFolder>> FIELD =
        new PropertyName<CqFieldValue<CqAttachmentFolder>>(PROPERTY_NAMESPACE, "field");

    /**
     * Returns the value of the {@link #FIELD FIELD} property as defined by this
     * proxy.
     * 
     * @return A CqFieldValue structure for the field that references this
     *         folder. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FIELD FIELD} property.
     */
    CqFieldValue<?> getField() throws WvcmException;

    /** The parent record of this folder. */
    PropertyName<CqRecord> RECORD =
        new PropertyName<CqRecord>(PROPERTY_NAMESPACE, "record");

    /**
     * Returns the value of the {@link #RECORD RECORD} property as
     * defined by this proxy.
     *
     * @return  A Record proxy for the record with which this folder is associated.
     *
     * @throws  WvcmException  if this Property instance does not define a value
     *                         for the {@link #RECORD RECORD} metadata
     *                         component.
     */
    CqRecord getRecord()
        throws WvcmException;

    /**
     * A CqRecordType proxy for the record-type resource that defines the parent
     * record of this attachment folder.
     */
    PropertyName<CqRecordType> RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE, "record-type");

    /**
     * A CqRecordType proxy for the record-type resource that defines what kind of
     * record this is.
     *
     * @return  The proxy for the record-type of this record is returned.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #RECORD_TYPE RECORD_TYPE} property.
     */
    CqRecordType getRecordType() throws WvcmException;
}
